//**************************************************************************
// The dd-GIF library Demo.
//
// This program demonstrates the funtions in the dd-GIF V1.0 Library.
//
// If you like the dd-GIF library and decide to use it for more than 30 days,
// you must register it. For more information regarding registration and the
// Software License, see DDGIF.DOC, included in this package.
//
//  TO COMPILE ->  Clipper demo.prg /N /W
//  TO LINK    ->  RTLink file demo lib ddgif, ddlib
//
//
// "GIF and 'Graphics Interchange Format' are trademarks (tm) of
//  CompuServe Incorporated, an H&R Block Company"
//
//  DD-GIF Library - (c) 1994, digital diversions, All Rights Reserved.
//
//                           digital diversions
//                          512 Corrinthia Court
//                      Elk Grove Village, IL  60007
//
// Enough legal-ease, lets move on...
//**************************************************************************
#include "box.ch"
#include "inkey.ch"
#include "directry.ch"

#include "dd.ch"  // dd-GIF definitions

Procedure Demo()
    Local nChoice := 1                      // The Menu Choice
    Local cSaveColor := SetColor("W/B")     // Set Screen Color
    Local GetList := {}                     // The Get list Array
    Local cScreenBuffer                     // Area to Save the Screen

    Set Scoreboard Off                      // Set up some Clipper defaults
    Set Message To 23 Center
    Set Wrap On
    Set Cursor Off

    //Lets display the COBRA BBS spot !
    If File("COBRA.GIF")
        ddSetMode(DD_VGA256)
        If ddShowGif("COBRA.GIF",,, DD_DARKESTBACKGROUND) == 0
            Inkey(0)
        Endif
        DDSetMode(DD_TEXT)
    Endif

    //Lets display the Logo screen
    If File("DDLOGO.GIF")
        ddSetMode(DD_VGA256)
        If ddShowGif("DDLOGO.GIF",,, DD_DARKESTBACKGROUND) == 0
            Inkey(5)
        Endif
        DDSetMode(DD_TEXT)
    Endif

    @ 0, 0, 24, 79 Box B_SINGLE + " "

    SayCenter(0, "dd-GIF Library V1.0 Demo")
    SetColor("W/B")
    SayCenter(1, "dd-GIF (c) 1994, digital diversions, All Rights Reserved.")

    SetColor("W+/B")
    SayCenter(6, "Please select the group of functions you wish to view:")

    Do While .T.
        @ 8, 28 Prompt "Display GIF Files       "     ;
                Message "Demonstrate the ddShowGif() function"
        @ 10,28 Prompt "Display Lines, and Boxes"     ;
                Message "Demonstrate the ddLine(), ddBox() and ddFill() functions"
        @ 12,28 Prompt "Display Fonts           "     ;
                Message "Demonstrate the ddLdFont() and ddSay() functions"
        @ 14,28 Prompt "Show all Functions      "     ;
                Message "Demonstrate all the functions at once"
        @ 16,28 Prompt "Exit this Demonstration"     ;
                Message "Return to the DOS prompt"

        Menu To nChoice

        Save Screen To cScreenBuffer

        Do Case
        Case nChoice == 1
            DoGIF()                         //Demo the GIF display functions
        Case nChoice == 2
            DoLINES()                       //Demo Lines and Boxes
        Case nChoice == 3
            DoFONTS()                       //Demo fonts
        Case nChoice == 4
            DoALL()                         //Demo everything at once.
        Case nChoice == 5
            Exit
        EndCase

        Restore Screen From cScreenBuffer
    Enddo

    Set Cursor On

    SetColor(cSaveColor)

    Clear Screen

    ? "Thank you for trying the dd-GIF Library Demo."
    ? " "
    ? " "

Return

//**************************************************************************
//**************************************************************************
Procedure DoGIF()
//**************************************************************************
//**************************************************************************
    Local aDirectory := {}, aGIFFiles := {} , nChoice := 0
    Local nCenterChoice := 0, aCenterOpts  := {"Centered", "Not Centered"}
    Local nBackChoice := 0, aBackOpts := {"Darkest", "Image Default"}
    Local nNumberColors, nImageTop, nImageLeft, nImageHeight, nImageWidth
    Local cGifFile, i, nScreenWidth, nScreenHeight, nBackParm
    Local GetList := {}, cScreenBuffer, cSaveColor
    Local RetVal

    @ 1,1 Clear To 23,78
    SayCenter(1, "Display GIF Files")
    Save Screen To cScreenBuffer

    cSaveColor = SetColor("W+/R")

    Do While .T.

        Set Cursor Off

        // Lets get all GIF files in the current directory.
        aDirectory = Directory("*.GIF")

        // Lets make sure that there are some files, if not apologize
        If Len(aDirectory) == 0
            Alert("Sorry, there are no GIF files in the current " + ;
                "directory. There were some sample GIF files included " + ;
                "with the dd-GIF package. Please copy them into the " + ;
                "current directory and try again.")
            Return
        Endif

        ASize(aGIFFiles, Len(aDirectory))
        For i = 1 to Len(aDirectory)
            aGIFFiles[i] = aDirectory[i, F_NAME]
        Next i
        ASort(aGIFFiles)

        // Ok, were cool, now lets display them in a box, and allow the
        // customer to select them.
        SayCenter(23, "Please select an Image to view, or Press ESC " + ;
                      "to return to the Main Menu")

        @ 4,32,16,47 Box B_DOUBLE + " "

        nChoice = AChoice(5, 34, 15, 46, aGIFFiles)

        // Check For Escape
        If nChoice == 0
            Exit
        Endif

        cGIFFile = aGIFFiles[nChoice]

        // Now Lets ask if the customer wants the image centered, or not.
        nCenterChoice = Alert("Do you want the image centered on the " + ;
                              "screen?", aCenterOpts)
        If nCenterChoice = 0
            Loop
        Endif

        // Now lets ask if the customer wants the darkest background
        // possible or use the image's default background (they may be
        // different).
        nBackChoice = Alert("Do you want the darkest background possible " + ;
                            "or display the image's default background?", ;
                            aBackOpts)
        If nBackChoice = 0
            Loop
        Endif

        // OK, now we're done harrassing the customer, lets get down to
        // business.
        // First lets find out how Many colors there are in the image
        // and set the Screen Max widht and height accordingly
        // (256 colors = 320x200 resolution)
        // (16 colors and less = 640x480 resolution)
        nNumberColors = ddGifInfo(cGIFFile, DDGIFINFO_COLORS)

        If nNumberColors == 256
            nScreenWidth = 320
            nScreenHeight = 200
        Else
            nScreenWidth = 640
            nScreenHeight = 480
        Endif

        // Get the Image's Height
        nImageHeight = ddGifInfo(cGIFFile, DDGIFINFO_HEIGHT)
        // Get the Image's Width
        nImageWidth = ddGifInfo(cGIFFile, DDGIFINFO_WIDTH)

        // If it weren't for the centering we would not need to concern
        // ourselves with width, height, left and top. ddShowGif would
        // automatically display the GIF image at the default coords
        // specified in the image's Header.

        If nCenterChoice = 1 // Customer wants image centered
            nImageTop = Int( (nScreenHeight / 2) - (nImageHeight / 2) )
            nImageLeft = Int( (nScreenWidth / 2) - (nImageWidth / 2) )
        Endif

        If nBackChoice = 1 //Customer wants darkest background
            nBackParm = DD_DARKESTBACKGROUND
        Else
            nBackParm = DD_DEFAULTBACKGROUND
        Endif

        //Set the approriate Video Mode
        If nNumberColors = 256
            ddSetMode(DD_VGA256)
        Else
            ddSetMode(DD_VGA16)
        Endif


        // Display the Picture
        If nCenterChoice = 1
            RetVal = ddShowGif(cGIFFile, nImageTop, nImageLeft, nBackParm, DD_FAST)
        Else
            RetVal = ddShowGIF(cGIFFile,,,nBackParm)
        Endif

        // check for GIF image errors or corruption
        If RetVal != DDGIF_OK
            // set the Video Mode back to text
            ddSetMode(DD_TEXT)
            Restore Screen From cScreenBuffer
            Tone(1000,2)
            // let the customer know what happened
            Alert("An error occurred when the GIF File was processed. The " + ;
                "Error Code was " + AllTrim(Str(RetVal)) + ". Most likely the " + ;
                "GIF image is corrupt.")
        Else
            // Wait for a key press
            Inkey(0)

            // Set the Video Mode back to Text
            ddSetMode(DD_TEXT)

            Restore Screen From cScreenBuffer
        Endif

    Enddo

    SetColor(cSaveColor)

Return



//**************************************************************************
//**************************************************************************
Procedure DOLines()
//**************************************************************************
//**************************************************************************
Local nStep := 10, nX1 := 0, nY1 := 0, nY2 := 0
Local nX2
Local nTop, nLeft, nHeight, nWidth, nLineWidth, nColor
Local i, cScreenBuffer

    // Save The Screen
    Save Screen to cScreenBuffer

    //Set the video mode for 640x480 16 Color
    ddSetMode(DD_VGA16)


    //Fill the whole screen with darkgrey
    ddFill(0, 0, 480, 640, DD_DARKGREY)

    //********************************************************************
    // Draw a pattern of lines
    //********************************************************************
    nX2 = (480 - 1) - ((480 - 1) % nStep)


    Do While nX1 <= nX2
        ddLine(nY1,nX1,nY2,nX2, DD_YELLOW)
        nX1 += nStep
        nY2 += nStep
    Enddo


    //********************************************************************
    // Draw a series of nested Hollow boxes
    // where the boxes are 3 pixels thick
    //********************************************************************
    // define the starting locations
    nTop = 100
    nLeft = 10
    nWidth = 200
    nHeight = 200

    // define how deep to nest each filled box
    nLineWidth = 3

    //define the starting color
    nColor = DD_BLUE                              // Color index 1

    // lets draw the boxes
    For i = 1 to nHeight / (nLineWidth + 1)
        ddBox(nTop, nLeft, nWidth, nHeight, nColor, nLineWidth)

        // Calc the next box
        nTop = nTop + nLineWidth + 1
        nLeft = nLeft + nLineWidth + 1
        nWidth = nWidth - ((nLineWidth + 1) * 2)
        nHeight = nHeight - ((nLineWidth + 1) * 2)

        // select the next color
        nColor++
        If nColor > 15
            nColor = 0
        Endif

    Next i


    //********************************************************************
    // Draw a series of filled boxes showing every color in the 16
    // Color Pallete
    //********************************************************************
    // define the starting locations
    nTop = 0
    nLeft = 485
    nWidth = 75
    nHeight = 55

    //define the starting color
    nColor = DD_BLACK                              // Color index 0

    // Draw the Boxes
    For i = 1 to 8
        // Draw the Left Box
        ddFill(nTop, nLeft, nHeight, nWidth, nColor)

        //Select the Next Color
        nColor++

        //Draw the Right Box
        ddFill(nTop, nLeft + nWidth + 5, nHeight, nWidth, nColor)

        // increment for the next row
        nTop = nTop + nHeight + 5

        // Select the Next Color
        nColor++
    Next i


    inkey(0)

    //Set the VideoMode to Text
    ddSetMode(DD_TEXT)


    // Restore The Screen
    Restore Screen From cScreenBuffer

    Return




//**************************************************************************
//**************************************************************************
Procedure DOFonts()
//**************************************************************************
//**************************************************************************
    Local aDirectory := {}, cFontFile, nChoice, aFontFiles := {}
    Local cFontMap, nFontLen, nFontname, cScreenBuffer, aText := {}
    Local i, RetVal, cSaveColor
    Local nFillTop, nFillHeight

    Set Cursor Off

    @ 1,1 Clear To 23,78
    SayCenter(1, "Display Fonts")
    Save Screen To cScreenBuffer

    cSaveColor = SetColor("W+/R")

    // The text to say on the graphics screen
    aText = {"ABCDEFGHIJKLMNOPQRSTUVWYXZ", "abcdefghijklmnopqrstuvwyxz", ;
             "0123456789`~!@#$%^&*()_+-={}|[]\:;<>,.?/", ;
             " ", ;
             '"To accomplish great things,', "we must not only plan,", ;
             "but act,", "not only dream,", 'but believe."', ;
             "   Anatole Franc"}



    Do While .T.

        // Set the Font buffer
        cFontMap = Space(4096)

        // Lets get all Font files in the current directory.
        aDirectory = Directory("*.FNT")

        // Lets make sure that there are some files, if not apologize
        If Len(aDirectory) == 0
            Alert("Sorry, there are no Font files in the current " + ;
                "directory. There were some sample Fonts included " + ;
                "with the dd-GIF package. Please copy them into the " + ;
                "current directory and try again.")
            Return
        Endif

        //Load the Font Filenames into an array
        ASize(aFontFiles, Len(aDirectory))
        For i = 1 to Len(aDirectory)
            aFontFiles[i] = aDirectory[i, F_NAME]
        Next i
        ASort(aFontFiles)

        // Ok, were cool, now lets display them in a box, and allow the
        // customer to select them.
        SayCenter(23, "Please select a Font to view, or Press ESC " + ;
                      "to return to the Main Menu")

        @ 4,32,17,47 Box B_DOUBLE + " "

        nChoice = AChoice(5, 34, 16, 46, aFontFiles)

        // Check For Escape
        If nChoice == 0
            Exit
        Endif

        cFontFile = aFontFiles[nChoice]

        // Load The Font
        nFontLen = ddLDFont(cFontFile, @cFontMap)
        // Check for errors
        If nFontLen < 0
            Alert ("There was any error loading the Font " + cFontFile + ;
                   "ddLDFont Return Value = " + AllTrim(Str(nFontLen)) )
            Return
        Endif

        // Trim the Font Map Down to the Size returned by ddLDFont()
        cFontMap = Left(cFontMap, nFontLen)

        // go into 640x480 mode
        ddSetMode(DD_VGA16)

        ddSay(0,0,15, cFontMap, AllTrim(cFontFile) +  ;
             "    640 x 480 mode ")

        for i = 1 to Len(aText)
            RetVal = ddSay(i, 0, DD_WHITE,  cFontMap, aText[i])
            if RetVal != 0
                ddSetMode(DD_TEXT)
                Alert("Error with ddSay, Return value = " + AllTrim(Str(retval)) )
                return
            endif
        next i

        inkey(0)

        // go into 320x200 mode
        ddSetMode(DD_VGA256)

        ddSay(0,0,DD_WHITE, cFontMap, AllTrim(cFontFile) +  ;
             "    320 x 200 mode ")

        for i = 1 to Len(aText)
            RetVal = ddSay(i, 0, 15, cFontMap, aText[i])
            if RetVal != 0
                ddSetMode(DD_TEXT)
                Alert("Error with ddSay, Return value = " + AllTrim(Str(retval)) )
                return
            endif
        next i

        inkey(0)

// Now lets display the some text with XOR on, this will cause the text
// to change colors when as it comes across different colors on the screen
        // go into 640x480 mode
        ddSetMode(DD_VGA16)

        ddSay(0,0,DD_YELLOW, cFontMap,  "The Following text shows the font capabilities " + ;
              "using XOR Color blending.")
        ddSay(2,0,DD_LIGHTCYAN, cFontMap, "The first text line does not have XOR on, but the " + ;
              "second line does.")
        ddSay(3,0,DD_LIGHTCYAN, cFontMap, "Notice how the first line of text disappears into " + ;
              "the colored box,")
        ddSay(4,0, DD_LIGHTCYAN, cFontMap, "whereas the second line of text changes color to " + ;
              "keep the contrast, ")
        ddSay(5,0, DD_LIGHTCYAN, cFontMap, "thereby keeping it visible.")

        //Lets calculate where to draw a filled box for this demonstration
        //if the FontMap size is 4096 then we are using a 8x16 font,
        //otherwise we are using an 8x8 font. Once we know this we can
        //calculate the line number to start drawing the filled box.
        //We want to figure out where text line 10 starts and end on
        //line 12.
        If Len(cFontMap) == 4096
            nFillTop = 10 * 16     // line 10 * 16 pixels
            nFillHeight  = 3 * 16  // fill 3 lines worth
        Else
            nFillTop = 10 * 8      // Line 10 * 8 pixels
            nFillHeight = 3 * 8    // fill 2 lines worth
        Endif

        // Now draw the filled box going over half the lines
        ddFill(nFillTop, 40, nFillHeight, 312, DD_YELLOW)

        // Now drop the Text with XOR OFF
        ddSay(10,0, DD_YELLOW, cFontMap, "This line demonstrates the XOR capability of Fonts.")


        // Now drop the same text on the next line with XOR ON, to turn on
        // XOR we add DD_XOR (256) to the color we want to display. We
        // Blink the line on and off, to do this we must erase the text
        // then re-write it.
        // With XORing this is simple, just keep re-writing the text,
        // it its already displayed, it will ber XOR'ed off, if it is not
        // displayed it will be XOR'ed on
        Do While .T.
            ddSay(12,0, DD_YELLOW + DD_XOR, cFontMap, "This line demonstrates the XOR capability of Fonts.")

            If Inkey(.7) != 0
                Exit
            Endif
        Enddo

        ddSetMode(DD_TEXT)

        Restore Screen From cScreenBuffer
    Enddo

    SetColor(cSaveColor)
Return





//**************************************************************************
//**************************************************************************
Procedure DoAll()
//**************************************************************************
//**************************************************************************
Local nHeight, nWidth, nTop, nLeft, nGifFound := .T., nRetVal
Local i, j, cFontMap, aBarTops := {100, 200, 300, 150, 350}
Local nFontLen, nDarkestColor
Local aBarColors := {DD_BLUE, DD_LIGHTGREEN, DD_RED, DD_YELLOW, DD_MAGENTA}
Local aXLabels := { "1989", "1990", "1991", "1992", "1993" }
Local aYLabels := { "800", "700", "600", "500", "400", "300", "200", "100", "  0"}


    //First lets see if the Logo is available, if not then warn the customer
    If !File("DD-GIF.GIF")
        Tone(1000,3)
        Alert("The GIF file 'DD-GIF.GIF' was not found! This GIF image " + ;
              "was provided with the dd-GIF package, please copy it into " + ;
              "the current directory and try again. This demonstration will " + ;
              "will continue regardless...")
        nGifFound = .F.
    Endif

    //Set the Video Mode for 640x480 16 colors
    ddSetMode(DD_VGA16)

    //Let's fill in the screen with bright white
    ddFill(0, 0, 480, 640, DD_WHITE)

    //Now let's display the Logo Centered at the top of the screen
    nWidth = ddGifInfo("DD-GIF.GIF", DDGIFINFO_WIDTH)
    nLeft = 320 - Int(nWidth / 2)
    ddShowGif("DD-GIF.GIF", 0, nLeft)

    // Get the darkest color in the Palette
    nDarkestColor = ddColor(DDCOLOR_DARKEST)

    //Now lets display a bar graph using lines, boxes, filled boxes and Fonts

    // First lets draw the baselines, starting with the Y baseline
    ddLine(100, 50, 420, 50, DD_BLACK)
    // Draw the X baseline
    ddLine(420, 50, 420, 624, DD_BLACK)

    // Lets draw the graph scale lines
    // Y scale
    For i = 100 to 420 Step 40
        ddLine(i, 45, i, 50, DD_BLACK)
    Next i
    // X Scale
    For i = 104 to 552 Step 112
        ddLine(420, i, 425, i, DD_BLACK)
    Next i

    // Now lets draw the bar outlines
    For i = 1 to 5
        ddBox(aBarTops[i], i * 112 - 25 - 8, 420 - aBarTops[i] + 1, 50, DD_BLACK)
    Next i

    //Now lets fill in the Boxes
    For i = 1 to 5
        ddFill(aBarTops[i] + 1, i * 112 - 25 - 8 + 1, 420 - aBarTops[i] - 1, 48, aBarColors[i])
    Next i

    // Now Lets drop some text

    //Load the font
    cFontMap = Space(4096)
    nFontLen = ddLdFont("Italic16.FNT", cFontMap)
    //Trim the Font map to the correct size
    cFontMap = Left(cFontMap, nFontLen)
    // Display the Title
    ddSay(5, 25, nDarkestColor, cFontMap, "Who-sa-whats-its Sales History")

    //Display the Y Labels
    nFontLen = ddLdFont("Thin_8.Fnt", cFontMap)
    cFontMap = Left(cFontMap, nFontLen)
    j = 1
    For i = 12 to 52 Step 5
        ddSay(i, 2, DD_BLUE, cFontMap, aYLabels[j])
        j++
    Next i

    //Display the XLabels
    j = 1
    For i = 11 to 67 Step 14
        ddSay(54, i, DD_BLUE, cFontMap, aXLabels[j])
        j++
    Next i

    //Wait for a keypress
    Inkey(0)
    //Set the video Mode back to text
    ddSetMode(DD_TEXT)

Return





//**************************************************************************
//**************************************************************************
// Display a String Centered on the Screen
Procedure SayCenter(nRow, cText)
//**************************************************************************
//**************************************************************************
    @ nRow, Int((MaxCol() / 2) - (Len(cText) / 2)) Say cText
Return




